<?php

namespace Drupal\messagebird;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Link;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use MessageBird\Exceptions\AuthenticateException;
use MessageBird\Exceptions\BalanceException;
use MessageBird\Exceptions\HttpException;
use MessageBird\Exceptions\RequestException;
use MessageBird\Exceptions\ServerException;

/**
 * Class MessageBirdServiceException.
 *
 * @package Drupal\messagebird
 */
class MessageBirdException implements MessageBirdExceptionInterface {
  use StringTranslationTrait;

  /**
   * MessageBird logger.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * MessageBird configuration.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * Current User object.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * MessageBirdException constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Configuration Factory object.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   Logger Channel Factory object.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   Account object.
   */
  public function __construct(ConfigFactoryInterface $config_factory, LoggerChannelFactoryInterface $logger_factory, AccountInterface $current_user) {
    $this->config = $config_factory->get('messagebird.settings');
    $this->logger = $logger_factory->get('messagebird');
    $this->currentUser = $current_user;
  }

  /**
   * Log the right status for the given Exception type.
   *
   * @param \Throwable $exception
   *   Exception Object.
   * @param array $args
   *   (optional) Arguments for t().
   */
  public function logError(\Throwable $exception, array $args = array()) {

    switch (TRUE) {
      case ($exception instanceof AuthenticateException):
        $this->logger->alert('Authentication failed, please check your credentials.');
        break;

      case ($exception instanceof BalanceException):
        $this->logger->warning('Balance: @error', array(
          '@error' => $exception->getMessage(),
        ));
        break;

      case ($exception instanceof HttpException):
        $this->logger->warning('HTTP: @error', array(
          '@error' => $exception->getMessage(),
        ));
        break;

      case ($exception instanceof RequestException):
        $this->logger->warning('Request: @error', array(
          '@error' => $exception->getMessage(),
        ));
        break;

      case ($exception instanceof ServerException):
        $this->logger->alert('Server: @error', array(
          '@error' => $exception->getMessage(),
        ));
        break;

      default:
        $this->logger->notice($exception->getMessage(), $args);
    }

    if ($this->config->get('debug.mode')) {
      $log_access = $this->currentUser->hasPermission('access site reports');
      $url = Link::createFromRoute(t('Recent log messages'), 'dblog.overview');
      $class = new \ReflectionClass($exception);

      $message = $this->t("MessageBird has thrown an @class_name, see 'url_to_dblog' for more info.", array(
        '@class_name' => $class->getShortName(),
        'url_to_dblog' => ($log_access) ? $url->toString() : t('Recent log messages'),
      ));

      drupal_set_message($message, 'warning');
    }
  }

}
