<?php

namespace Drupal\messagebird;

use Drupal\Core\Config\ConfigFactoryInterface;
use MessageBird\Objects\Lookup;

/**
 * Class MessagebirdServiceLookup.
 *
 * @package Drupal\messagebird
 */
class MessageBirdLookup implements MessageBirdLookupInterface {

  /**
   * MessageBird Client object.
   *
   * @var \MessageBird\Client
   */
  protected $client;

  /**
   * MessageBird Exception object.
   *
   * @var \Drupal\messagebird\MessageBirdExceptionInterface
   */
  protected $exception;

  /**
   * MessageBird Configuration.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;

  /**
   * MessageBirdLookup constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config
   *   Config Factory object.
   * @param \Drupal\messagebird\MessageBirdClientInterface $client
   *   MessageBird Client object.
   * @param \Drupal\messagebird\MessageBirdExceptionInterface $exception
   *   MessageBird Exception object.
   */
  public function __construct(ConfigFactoryInterface $config, MessageBirdClientInterface $client, MessageBirdExceptionInterface $exception) {
    $this->client = $client->getClient();
    $this->exception = $exception;
  }

  /**
   * Look up a phone number.
   *
   * @param string $number
   *   Phone number, accepts multiple formats.
   * @param string|null $country_code
   *   (optional) ISO 3166-2 country code.
   *
   * @return \MessageBird\Objects\Lookup|null
   *   Full Lookup object or NULL on failure.
   */
  public function lookupNumber($number, $country_code = NULL) {
    $lookup = NULL;

    try {
      /** @var Lookup $lookup */
      $lookup = $this->client->lookup->read($number, $country_code);
    }
    catch (\Exception $e) {
      $this->exception->logError($e);
    }
    finally {
      return $lookup;
    }
  }

  /**
   * Get the full MessageBird Lookup object.
   *
   * @param \MessageBird\Objects\Lookup $lookup
   *   Lookup object.
   * @param string $format
   *   Preferred phone number format, defaults to numbers only.
   *
   * @return string
   *   The preferred formatted phone number or empty on failure.
   */
  public function getPhoneNumber(Lookup $lookup, $format = NULL) {
    if (is_null($format)) {
      return (string) $lookup->getPhoneNumber();
    }

    $formats = $lookup->getFormats();
    if (isset($formats->{$format})) {
      return (string) $formats->{$format};
    }

    // Prepare exception.
    $exception = new \Exception('Current lookup format %format isn\'t supported, possible options are: %options.');
    $args = array(
      '%format' => $format,
      '%options' => implode(', ', array_keys((array) $formats)),
    );

    $this->exception->logError($exception, $args);

    return '';
  }

}
