<?php

namespace Drupal\messagebird;

use Drupal\Core\Config\ConfigFactoryInterface;
use MessageBird\Objects\Message;

/**
 * Class MessageBirdMessage.
 *
 * @package Drupal\messagebird
 */
class MessageBirdMessage implements MessageBirdMessageInterface {

  /**
   * MessageBird Client.
   *
   * @var \MessageBird\Client
   */
  protected $client;

  /**
   * MessageBird configuration.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * MessageBird Exception object.
   *
   * @var \Drupal\messagebird\MessageBirdExceptionInterface
   */
  protected $exception;

  /**
   * MessageBirdMessage constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Configuration Factory object.
   * @param \Drupal\messagebird\MessagebirdClientInterface $client
   *   MessageBird Client object.
   * @param \Drupal\messagebird\MessageBirdExceptionInterface $exception
   *   MessageBird Exception object.
   */
  public function __construct(ConfigFactoryInterface $config_factory, MessagebirdClientInterface $client, MessageBirdExceptionInterface $exception) {
    $this->client = $client->getClient();
    $this->exception = $exception;
    $this->config = $config_factory->get('messagebird.settings');
  }

  /**
   * Send a SMS message through MessageBird.
   *
   * @param string $body
   *   The text to be send.
   * @param string $recipients
   *   One or more recipients, comma separated.
   * @param string|null $originator
   *   (optional) Originator of the Message.
   * @param array $options
   *   (optional) Extra options to send with the Message.
   *
   * @return \MessageBird\Objects\Message
   *   MessageBird object.
   */
  public function sendMessage($body, $recipients, $originator = NULL, $options = array()) {
    if (empty($originator)) {
      $originator = $this->config->get('default.originator');
    }

    try {
      $message = new Message();
      $message->originator = $originator;
      $message->recipients = $recipients;
      $message->body = $body;

      // Append the optional parameters.
      foreach ($options as $parameter => $value) {
        if (!empty($parameter) && !empty($value)) {
          $message->{$parameter} = $value;
        }
      }

      // Send the message to the recipient(s).
      $this->client->messages->create($message);

      if ($this->config->get('debug.mode')) {
        $debug_params = array(
          'Body' => $body,
          'Recipients' => $recipients,
          'Originator' => $originator,
        );

        foreach ($debug_params as $key => $value) {
          drupal_set_message(t('@messagebird_debug_key: @messagebird_debug_value', array(
            '@messagebird_debug_key' => $key,
            '@messagebird_debug_value' => $value,
          )));
        }
      }
    }
    catch (\Exception $e) {
      $this->exception->logError($e);
    }
    finally {
      return $this->client->messages->getObject();
    }
  }

}
